@@ -4,14 +4,14 @@ use warnings;
 
 package Git::Wrapper;
 #ABSTRACT: Wrap git(7) command-line interface
-$Git::Wrapper::VERSION = '0.038';
+$Git::Wrapper::VERSION = '0.047';
 our $DEBUG=0;
 
 # Prevent ANSI color with extreme prejudice
 # https://github.com/genehack/Git-Wrapper/issues/13
 delete $ENV{GIT_PAGER_IN_USE};
 
-use File::pushd;
+use File::chdir;
 use File::Temp;
 use IPC::Open3      qw();
 use Scalar::Util    qw(blessed);
@@ -83,7 +83,7 @@ sub RUN {
   my( @out , @err );
 
   {
-    my $d = pushd $self->dir unless $cmd eq 'clone';
+    local $CWD = $self->dir unless $cmd eq 'clone';
 
     my ($wtr, $rdr, $err);
 
@@ -168,12 +168,23 @@ sub has_git_in_path {
 sub log {
   my $self = shift;
 
+  if ( grep /format=/, @_ ) {
+    die Git::Wrapper::Exception->new(
+      error  => [qw/--format not allowed. Use the RUN() method if you with to use a custom log format./],
+      output => undef,
+      status => 255 ,
+    );
+  }
+
   my $opt  = ref $_[0] eq 'HASH' ? shift : {};
   $opt->{no_color}         = 1;
   $opt->{pretty}           = 'medium';
+  $opt->{no_abbrev}        = 1;  # https://github.com/genehack/Git-Wrapper/issues/67
 
   $opt->{no_abbrev_commit} = 1
     if $self->supports_log_no_abbrev_commit;
+  $opt->{no_expand_tabs} = 1
+    if $self->supports_log_no_expand_tabs;
 
   my $raw = defined $opt->{raw} && $opt->{raw};
 
@@ -211,7 +222,12 @@ sub log {
     if ($raw) {
       my @modifications;
 
-      while(@out and $out[0] =~ m/^\:(\d{6}) (\d{6}) (\w{7})\.\.\. (\w{7})\.\.\. (\w{1})\t(.*)$/) {
+      # example outputs:
+      #  regular:
+      # :000000 100644 0000000000000000000000000000000000000000 ce013625030ba8dba906f756967f9e9ca394464a A     foo/bar
+      #  with score value after file type (see https://github.com/genehack/Git-Wrapper/issues/70):
+      # :100644 100644 c659037... c659037... R100       foo bar
+      while(@out and $out[0] =~ m/^\:(\d{6}) (\d{6}) (\w{40}) (\w{40}) (\w{1}[0-9]*)\t(.*)$/) {
         push @modifications, Git::Wrapper::File::RawModification->new($6,$5,$1,$2,$3,$4);
         shift @out;
       }
@@ -219,6 +235,9 @@ sub log {
     }
 
     push @logs, $current;
+
+    last unless @out; # handle running out of log
+    shift @out unless $out[0] =~ /^commit/;  # blank line at end of entry, except merge commits;
   }
 
   return @logs;
@@ -275,6 +294,14 @@ sub supports_log_no_abbrev_commit {
   return ( versioncmp( $self->version , '1.7.6' ) eq -1 ) ? 0 : 1;
 }
 
+sub supports_log_no_expand_tabs {
+  my $self = shift;
+
+  # The '--no-expand-tabs' option to git log was added in version 2.9.0
+  return 0 if ( versioncmp( $self->version , '2.9' ) eq -1 );
+  return 1;
+}
+
 sub supports_log_raw_dates {
   my $self = shift;
 
@@ -388,7 +415,7 @@ Git::Wrapper - Wrap git(7) command-line interface
 
 =head1 VERSION
 
-version 0.038
+version 0.047
 
 =head1 SYNOPSIS
 
@@ -412,7 +439,7 @@ argument passing, instead of CLI-style C<--options> as L<Git> does.
 
 Except as documented, every git subcommand is available as a method on a
 Git::Wrapper object. Replace any hyphens in the git command with underscores
-(for example, 'git init-db' would become C<$git->init_db>).
+(for example, C<git init-db> would become C<< $git->init_db >>).
 
 =head2 Method Arguments
 
@@ -456,7 +483,7 @@ invokes the command line
 
 I<N.b.> Because of the way arguments are parsed, should you need to pass an
 explicit '0' value to an option (for example, to have the same effect as
-C<--abrrev=0> on the command line), you should pass it with a leading space, like so:
+C<--abbrev=0> on the command line), you should pass it with a leading space, like so:
 
   $git->describe({ abbrev => ' 0' };
 
@@ -480,25 +507,85 @@ yourself before passing it to C<Git::Wrapper> methods.
 =head2 Error handling
 
 If a git command exits nonzero, a C<Git::Wrapper::Exception> object will be
-thrown (via C<die()>). These objects have three useful methods:
+thrown (via C<die>) and may be capture via C<eval> or L<Try::Tiny>, for
+example.
+
+The error object has three useful methods:
 
 =over
 
 =item * error
 
-error message
+Returns the full error message reported by the resulting git command sent to
+C<STDERR>. This method should not be used as a success/failure check, as
+C<git> will sometimes produce output on STDERR when a command is successful.
 
 =item * output
 
-normal output, as a single string
+Returns the full output generated by the git command that is sent to
+C<STDOUT>.  This method should not be used as a success/failure check, as
+C<git> will frequently not have any output with a successful command.
 
 =item * status
 
-the exit status
+Returns the non-zero exit code reported by git on error
 
 =back
 
-These exception objects stringify to the error message.
+=head3 Using Try::Tiny
+
+L<Try::Tiny> is the recommended way to catch exception objects thrown by
+L<Git::Wrapper>.
+
+  use Try::Tiny
+
+  my $git = Git::Wrapper->new('/path/to/my/repo');
+
+  try {
+    # equivalent to, "git --non-existent-option=1" on the commandline
+    $git->status({ "non-existent-option"=>1 });
+  }
+  catch {
+    # print STERR from erroneous git command
+    print $_->error;
+
+    # print STOUT from git command
+    print $_->output;
+
+    # print non-zero exist status of git processo
+    print $_->status;
+
+    # quotes are overloaded, so:
+    print "$_"; # equivalent to $_->error
+  };
+
+=head3 Using C<eval>
+
+If for some reason you are unable to use L<Try::Tiny>, it is also possible to
+use the C<eval> function to catch exception objects. B<THIS IS NOT
+RECOMMENDED!>
+
+  my $git = Git::Wrapper->new('/path/to/my/repo');
+
+  my $ok = eval {
+    # equivalent to, "git --non-existent-option=1" on the commandline
+    $git->status({ "non-existent-option"=>1 });
+    1;
+  };
+
+  if ($@ and ref $@ eq q{Git::Wrapper::Exception}) {
+    # print STERR from erroneous git command
+    print $@->error;
+
+    # print STOUT from git command
+    print $@->output;
+
+    # print non-zero exist status of git processo
+    print $@->status;
+
+    # quotes are overloaded, so:
+    print "$@"; # equivalent to $@->error
+  }
 
 =head1 METHODS
 
@@ -535,7 +622,9 @@ you want ANSI color highlighting, you'll need to bypass via the RUN() method
   my @logs = $git->log;
 
 Instead of giving back an arrayref of lines, the C<log> method returns a list
-of C<Git::Wrapper::Log> objects. They have four methods:
+of C<Git::Wrapper::Log> objects.
+
+There are five methods in a C<Git::Wrapper::Log> objects:
 
 =over
 
@@ -547,8 +636,46 @@ of C<Git::Wrapper::Log> objects. They have four methods:
 
 =item * message
 
+=item * modifications
+
+Only populated with when C<< raw => 1 >> option is set; see L<Raw logs> below.
+
 =back
 
+=head3 Raw logs
+
+Calling the C<log> method with the C<< raw => 1 >> option set, as below, will
+do additional parsing to populate the C<modifications> attribute on each
+C<Git::Wrapper::Log> object. This method returns a list of
+C<Git::Wrapper::File::RawModification> objects, with can be used to get
+filenames, permissions, and other metadata associated with individual files in
+the given commit. A short example, to loop over all commits in the log and
+print the filenames that were changed in each commit, one filename per file:
+
+    my @logs = $git->log({ raw => 1 });
+    foreach my $log ( @logs ) {
+        say "In commit '" . $log->id . "', the following files changed:";
+        my @mods = $log->modifications;
+        foreach my $mod ( @mods ) {
+            say "\t" . $mod->filename;
+        }
+    }
+
+Note that some commits (e.g., merge commits) will not contain any file
+changes. The C<modifications> method will return an empty list in that case.
+
+=head3 Custom log formats
+
+C<log> will throw an exception if it is passed the C<--format> option. The
+reason for this has to do with the fact that the parsing of the full log
+output into C<Git::Wrapper::Log> objects assumes the default format provided
+by `git` itself. Passing C<--format> to the underlying `git log` method affects
+this assumption and the output is no longer able to be processed as intented.
+
+If you wish to specify a custom log format, please use the L<RUN> method
+directly.  The caller will be supplied with the full log output. From there,
+the caller may process the output as it wishes.
+
 =head2 has_git_in_path
 
 This method returns a true or false value indicating if there is a 'git'
@@ -558,14 +685,17 @@ binary in the current $PATH.
 
 =head2 supports_log_no_abbrev_commit
 
+=head2 supports_log_no_expand_tabs
+
 =head2 supports_log_raw_dates
 
 =head2 supports_hash_object_filters
 
 These methods return a true or false value (1 or 0) indicating whether the git
 binary being used has support for these options. (The '--porcelain' option on
-'git status', the '--no-abbrev-commit' and '--date=raw' options on 'git log',
-and the '--no-filters' option on 'git hash-object' respectively.)
+'git status', the '--no-abbrev-commit', '--no-expand-tabs', and '--date=raw'
+options on 'git log', and the '--no-filters' option on 'git hash-object'
+respectively.)
 
 These are primarily for use in this distribution's test suite, but may also be
 useful when writing code using Git::Wrapper that might be run with different
@@ -699,7 +829,7 @@ methods described above (i.e., C<log>, C<status>, etc.). This can be useful
 in various situations, such as when you want to produce a particular log
 output format that isn't compatible with the way C<Git::Wrapper> constructs
 C<Git::Wrapper::Log>, or when you want raw C<git status> output that isn't
-parsed into a <Git::Wrapper::Status> object.
+parsed into a C<Git::Wrapper::Status> object.
 
 This method should be called with an initial string argument of the C<git>
 subcommand you want to run, followed by a hashref containing options and their
@@ -715,6 +845,9 @@ values, and then a list of any other arguments.
     # while 'RUN('log')' returns an array of chomped lines
     my @log_lines = $git->RUN('log');
 
+    # getting the full of commit SHAs via `git log` by using the '--format' option
+    my @log_lines = $git->RUN('log', '--format=%H');
+
 =head2 ERR
 
 After a command has been run, this method will return anything that was sent
